package POWER::Event::IO::SSL::SERVER;

use version; $VERSION = qv('0.3.2');    # update POD & Changes & README

use warnings;
use strict;
use Carp;

use Crypt::MatrixSSL 1.83;

use base qw( POWER::Event::IO::SSL );
use POWER::Event::IO::share;


#     SSL  .
sub new {
    my ($class, $opt_ref) = @_;
    my $self = bless {
        # Set default values.
        ssl_crt        => undef,# filename(s) with server certificate(s)
        ssl_key        => undef,# filename with server private key
        ssl_key_pass   => undef,# password to decrypt private key
        _ssl           => undef,# MatrixSSL 'session' object
        _ssl_keys      => undef,# MatrixSSL 'keys' object
        _ssl_handshaked=> 0,    # flag, will be true after handshake
                                # was matrixSslEncoded()'d and
                                # {out_buf} will be 100% sent.
        # must have:
        in_buf_limit=> undef,
        in_buf      => q{},
        in_bytes    => 0,
        out_pos     => undef,
        out_buf     => q{},
        out_bytes   => 0,
        }, $class;
    # Load user values.
    for my $k ( grep {m/\A(?:ssl_crt|ssl_key|ssl_key_pass)\z/xms}
                keys %$opt_ref ) {
        $self->{$k} = $opt_ref->{$k};
    }
    croak '{ssl_crt} and {ssl_key} required'
        if !defined $self->{ssl_crt} || !defined $self->{ssl_key};
    # Initialize SSL.
    # TODO OPTIMIZATION Cache {_ssl_keys}.
    matrixSslReadKeys($self->{_ssl_keys}, $self->{ssl_crt}, $self->{ssl_key},
        $self->{ssl_key_pass}, undef)
        == 0 or croak 'matrixSslReadKeys: wrong {ssl_crt}, {ssl_key} or {ssl_key_pass}?';
    matrixSslNewSession($self->{_ssl}, $self->{_ssl_keys},
        undef, $SSL_FLAGS_SERVER)
        == 0 or croak 'matrixSslNewSession';
    return $self;
}


1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Event::IO::SSL::SERVER - ssl server plugin for POWER::Event::IO


=head1 VERSION

This document describes POWER::Event::IO::SSL::SERVER version 0.3.2


=head1 SYNOPSIS

See L<POWER::Event::IO/SYNOPSIS>.

=head1 DESCRIPTION

See L<POWER::Event::IO/PLUGINS>.

=head1 INTERFACE

=over

=item B<new( \%PARAMS )>

    ( POWER::Event::IO).

 %PARAMS     ,     
   :

    :
  ssl_crt + ssl_key
    :
  ssl_key_pass

  ,       
. L</Public FIELDS>.

     [A-Z].    
     new()      .

  .


=back

=head1 Public FIELDS

        
      .

       B<RO>   
  B<RW> ,     .

          
 _     (     
    )    [A-Z]
  (     new() /
//   ).


=over

=item B<ssl_crt>        I<RO>    new()

=item B<ssl_key>        I<RO>    new()

=item B<ssl_key_pass>   I<RO>     new()

ssl_crt  ssl_key        
  private key ( PEM ).
 private key ,   ssl_key_pass    
.

       ,    ssl_crt
      Σ  ';'.


=back

=head1 TIMEOUTS

    . L<POWER::Event::IO/TIMEOUTS>.

=over

=item $POWER::Event::IO::SSL::TO_SSL_HANDSHAKE

   SSL handshake.   
POWER::Event::IO::SSL::CLIENT  POWER::Event::IO::SSL::SERVER
      POWER::Event::IO.

  30 .   : "ssl handshake timed out".


=back

=head1 REALIZATION

  SSL   Crypt::MatrixSSL.


=head1 DIAGNOSTICS

=head2 EXCEPTIONS

=over

=item C<< matrixSsl... >>

 MatrixSSL (     trusted_CA
 ssl_crt, etc...).


=back

=head1 DEPENDENCIES

 Crypt::MatrixSSL 1.83

 POWER::Event::IO


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2006,2008 Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.


