package POWER::Event::IO::STREAM;

use version; $VERSION = qv('0.2.0');    # update POD & Changes & README

use warnings;
use strict;
use Carp;

use Scalar::Util qw( weaken dualvar );
use Socket;

use POWER::Event::IO::share;
use POWER::Epoll;


our $Stream = bless {}, __PACKAGE__;        # Singleton.

our $TO_CONNECT = int 30 / $TICK_RATE + 0.5;# default timeout for connect
our $TO_WRITE   = int 30 / $TICK_RATE + 0.5;# default timeout for write

my (@Timer_connect, @Timer_write);

push @Tick_handlers, \&_tick;

# WARNING      _err!    - , 
#              -  : connection reset by peer
#            ssl handshake timeout (). ,   ,  
#                .
sub _tick {
    for my $io (@{ shift @Timer_connect || [] }) {
        next unless $io;
        next if $io->{_state} != $CONNECTING;
        $io->{_events}         |= $ERR;
        $io->{_err}             = dualvar(-1, q{connecting timed out});
        $Ready{ $io->{fd} }    = $io;
    }
    for my $io (@{ shift @Timer_write || [] }) {
        next unless $io;
        #       Σ
        if ($io->{_epoll} & $EPOLLOUT) {
            #  tick,     
            my $to_tick = $io->{_tick_write} + $TO_WRITE + 1;
            #        -  
            # ԣ       $io->close(), 
            #          (..
            # _timer_write  )
            if ($to_tick <= $Tick) {
                $io->{_events}         |= $ERR;
                $io->{_err}             = dualvar(-1, q{writing timed out});
                $Ready{ $io->{fd} }     = $io;
            }
            #   ,     $Tick==$to_tick
            else {
                unshift @{ $Timer_write[ $to_tick - $Tick - 1 ] }, $io;
                weaken     $Timer_write[ $to_tick - $Tick - 1 ][0];
            }
        }
        #      ,   
        # EAGAIN    _write()
        else {
            $io->{_timer_write} = 0;
        }
    }
}

sub CONFIG {
    my ($self, $io) = @_;
    #       ..   
    #  ,       
    #  EPOLLERR.
    connect($io->{fh}, sockaddr_in($io->{port}, inet_aton($io->{ip})));
    #  host    DNS ,   _real_config
    #     ,      epoll_add().
    epoll_add($EPFD, $io->{fd}, $io->{_epoll})
        if defined $io->{fd}; # XXX        epoll_add?
    $io->{_state} = $CONNECTING;
    unshift @{ $Timer_connect[$TO_CONNECT] }, $io;
    weaken     $Timer_connect[$TO_CONNECT][0];
    return;
}

#  EINTR (  sysread/syswrite)  
#     EINTR      fh! :-/ 
# EINTR       .

sub READ {
    my ($self, $io) = @_;
    # WARNING     LT epoll  EOF?   
    #           EPOLLIN       EOF?
    #  ...    ?
    my $n = sysread $io->{fh}, $_[2], 4096, length $_[2];
    if (defined $n) {
        # $EOF     .
        if (!$n && !$io->{is_eof}) {
            $io->{is_eof}   = 1;
            $io->{_events} |= $EOF;
        }
    }
    elsif ($! != $EAGAIN) {
        $io->{_err}         = $!;
        $io->{_events}     |= $ERR;
    }
    return $n||0;
}

sub WRITE {
    my ($self, $io, undef, $out_pos) = @_;
    my $len = length $_[2];
    # work around 'Offset outside string' error in syswrite
    if ($len <= $out_pos) {
        return 0;
    }
    #  ...   ...    ?
    my $n = syswrite $io->{fh}, $_[2], 0xFFFF, $out_pos;
    if (defined $n) {
        # die 'Internal error' if !$n;
        #  ,    ,   
        # _tick_write        $TO_WRITE
        #   __ .
        $io->{_tick_write} = $Tick;
        if ($len == ($n + $out_pos)) {
            if ($io->{_epoll} & $EPOLLOUT) {
                $io->{_epoll} &= ~$EPOLLOUT;
                epoll_mod($EPFD, $io->{fd}, $io->{_epoll});
            }
        } else {
            EPOLLOUT:
            if (!($io->{_epoll} & $EPOLLOUT)) {
                $io->{_epoll} |= $EPOLLOUT;
                epoll_mod($EPFD, $io->{fd}, $io->{_epoll});
            }
            #      -   .
            if (!$io->{_timer_write}) {
                $io->{_timer_write} = 1;
                unshift @{ $Timer_write[$TO_WRITE] }, $io;
                weaken     $Timer_write[$TO_WRITE][0];
            }
        }
    }
    elsif ($! != $EAGAIN) {
        $io->{_err}           = $!;
        $io->{_events}       |= $ERR;
    }
    else {
        goto EPOLLOUT;
    }
    return $n||0;
}


1; # Magic true value required at end of module
__END__

=head1 DESCRIPTION

See L<POWER::Event::IO>.

