package POWER::Event::IO::share;

use version; $VERSION = qv('0.2.0');    # update POD & Changes & README

use warnings;
use strict;
use Carp;

use Scalar::Util qw( weaken dualvar );
use Errno qw( EAGAIN EINTR EINPROGRESS );
use Socket;

use POWER::Epoll 2.0.0;
use POWER::Event::Timer;

# %Class        -    
# %Ready        - ,     callback
# $Tick         - how many times _tick_tack() was called
# $DNS          -  DNS-
# @READ         - ,      _read
# @WRITE        - ,      _write
use vars qw(
    %Class %Ready $DNS $EPFD @READ @WRITE
    $Tick $TICK_RATE @Tick_handlers

    $EINBUFLIMIT
    $EAGAIN $EINTR $EINPROGRESS
    $AF_INET $PROTO_TCP $PROTO_UDP $SOCK_STREAM $SOCK_DGRAM
    $SOL_SOCKET $SO_REUSEADDR $REUSEADDR_VAL $SOMAXCONN
    $FIONBIO $IOCTL_TRUE
    $EPOLLIN $EPOLLOUT $EPOLLHUP $EPOLLERR $EPOLLPRI
    
    $ERR $RESOLVED $CONNECTED $ACCEPTED $IN $OUT $EOF $SENT
    $RESOLVING $CONNECTING $HANDLING
    $STREAM_IO $PACKET_IO
);


BEGIN {
    $EPFD = epoll_create(100);

    $Tick = 0;
    *TICK_RATE      = \0.1;         # timer rate (in float seconds)
                                    # ALL TIMEOUTS BELOW IN "ticks" AND >= 1 !
    $EINBUFLIMIT    = dualvar(-100, 'in_buf_limit reached');

    # Cache for speed:
    *EAGAIN         = \EAGAIN;
    *EINTR          = \EINTR;
    *EINPROGRESS    = \EINPROGRESS;
    *PROTO_TCP      = \getprotobyname('tcp');
    *PROTO_UDP      = \getprotobyname('udp');
    *AF_INET        = \AF_INET;
    *SOCK_STREAM    = \SOCK_STREAM;
    *SOCK_DGRAM     = \SOCK_DGRAM;
    *SOL_SOCKET     = \SOL_SOCKET;
    *SO_REUSEADDR   = \SO_REUSEADDR;
    *REUSEADDR_VAL  = \pack('L!', 1);
    *SOMAXCONN      = \SOMAXCONN;
    *FIONBIO        = \0x5421;
    $IOCTL_TRUE     = "\1\0\0\0";   # ioctl()     
    *EPOLLIN        = \EPOLLIN;
    *EPOLLOUT       = \EPOLLOUT;
    *EPOLLHUP       = \EPOLLHUP;
    *EPOLLERR       = \EPOLLERR;
    *EPOLLPRI       = \EPOLLPRI;

    # :
    *ERR            = \(1<<0);
    *RESOLVED       = \(1<<1);
    *CONNECTED      = \(1<<2);
    *ACCEPTED       = \(1<<3);
    *IN             = \(1<<4);
    *OUT            = \(1<<5);
    *EOF            = \(1<<6);
    *SENT           = \(1<<7);

    # :
    *RESOLVING      = \(1<<0);
    *CONNECTING     = \(1<<1);
    *HANDLING       = \(1<<2);

    # :
    *STREAM_IO      = \(1<<0);
    *PACKET_IO      = \(1<<1);

}


sub import {
    my $pkg = caller(0);
    no strict 'refs';
    *{$pkg.'::'.$_} = \${$_} for qw(
        EPFD DNS Tick TICK_RATE
    
        EINBUFLIMIT
        EAGAIN EINTR EINPROGRESS
        PROTO_TCP PROTO_UDP AF_INET SOCK_STREAM SOCK_DGRAM
        SOL_SOCKET SO_REUSEADDR REUSEADDR_VAL SOMAXCONN
        FIONBIO IOCTL_TRUE
        EPOLLIN EPOLLOUT EPOLLHUP EPOLLERR EPOLLPRI

        ERR RESOLVED CONNECTED ACCEPTED IN OUT EOF SENT
        RESOLVING CONNECTING HANDLING
        PACKET_IO STREAM_IO
        );
    *{$pkg.'::'.$_} = \@{$_} for qw(
        READ WRITE Tick_handlers
        );
    *{$pkg.'::'.$_} = \%{$_} for qw(
        Class Ready
        );
}


set_timer({ every => $TICK_RATE, cb => \&_tick_tack });

#    :
sub _tick_tack {
    $Tick++;
    $_->() for @Tick_handlers;
}


1; # Magic true value required at end of module
__END__

=head1 DESCRIPTION

See L<POWER::Event::IO>.
