use warnings;
use strict;
BEGIN { require 't/.lib.t' }

my $USUAL = '/tmp';
my $NETFS = $Build->notes('netfs');
my $TMPFS = '/dev/shm';

END {
    system("rm -f \Q$USUAL\E/_type_.{wr,rd}");
    system("rm -f \Q$NETFS\E/_type_.{wr,rd}") if length $NETFS;
    system("rm -f \Q$TMPFS\E/_type_.{wr,rd}");
}

my $ENOTPERM = "epoll_add: Operation not permitted\n";
@CheckPoint = (
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "> $USUAL/_type_.wr: not pollable",
    [ 'is_poll',    3,                  ], '... can write to file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "< $USUAL/_type_.rd: not pollable",
    [ 'is_poll',    'z',                ], '... can read from file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "> $NETFS/_type_.wr: not pollable",
    [ 'is_poll',    3,                  ], '... can write to file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "< $NETFS/_type_.rd: not pollable",
    [ 'is_poll',    'z',                ], '... can read from file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "> $TMPFS/_type_.wr: not pollable",
    [ 'is_poll',    3,                  ], '... can write to file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "< $TMPFS/_type_.rd: not pollable",
    [ 'is_poll',    'z',                ], '... can read from file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "< /dev/zero: not pollable",
    [ 'is_poll',    "\x00",             ], '... can read from file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "< /dev/null: not pollable",
    [ 'is_poll',    q{},                ], '... can read from file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "> /dev/null: not pollable",
    [ 'is_poll',    0,                  ], '... can write to file',
    [ 'is_poll',    'OK'                ], "< /dev/random: pollable",
    [ 'is_poll',    'random'            ], '... can read from file',
    [ 'is_poll',    'DIE',  $ENOTPERM   ], "< /dev/urandom: not pollable",
    [ 'is_poll',    'random'            ], '... can read from file',
);
plan tests => @CheckPoint/2 + 2;

sub cb {
    my ($io) = @_;
  &checkpoint;
    die 'cb done';
}

sub is_poll {
    my ($perm, $path) = @_;
    my ($io, $fh, $buf);
    # create file to read from
    if ($perm eq '<') {
        system("[ -w `dirname \Q$path\E` ] && echo zxc > \Q$path\E");
    }
    open $fh, $perm, $path              or die "open: $!";
    # check ability to read/write
    if ($perm eq '>') {
        print $fh 'abc'                 or die "print $!";
    } else {
        defined sysread $fh, $buf, 1    or die "sysread: $!";
    }
    # check is POWER::Event::IO support this fh
    Nonblock($fh);
    eval {
        $io = POWER::Event::IO->new({ fh=>$fh, cb=>\&cb, wait_for=>$IN|$OUT })
    };
    if ($@) {
        close $fh                       or die "close: $!";
        checkpoint('DIE', $@);
    } else {
        $io->close();
        checkpoint('OK');
    }
    # after close, check result of read/write operation above
    if ($perm eq '>') {
        checkpoint(-s $path);
    } else {
        $buf = 'random' if $path =~ /random/;
        checkpoint($buf);
    }
}

# usual file
is_poll('>', "$USUAL/_type_.wr");
is_poll('<', "$USUAL/_type_.rd");

# file on network file system
SKIP: {
  my $skip_reason = !length $NETFS  ? 'no network fs available'
                  : !-w $NETFS      ? 'network fs not writable'
                  :                   undef
                  ;
  if ($skip_reason) {
    splice @CheckPoint, 0, 4*2;
    skip $skip_reason, 4;
  }
  is_poll('>', "$NETFS/_type_.wr");
  is_poll('<', "$NETFS/_type_.rd");
}


# file on tmpfs
is_poll('>', "$TMPFS/_type_.wr");
is_poll('<', "$TMPFS/_type_.rd");

# file on devfs(udev)
is_poll('<', "/dev/zero");
is_poll('<', "/dev/null");
is_poll('>', "/dev/null");
diag "Testing /dev/random. If test hangs, please move your mouse. :)";
is_poll('<', "/dev/random");
is_poll('<', "/dev/urandom");

# directory
mkdir "$USUAL/_type_.dir"               or die "mkdir: $!";
opendir my $DIR, "$USUAL/_type_.dir"    or die "opendir: $!";
defined(my $DIR_buf = readdir $DIR)     or die "readdir: $!";
is(fileno($DIR), undef,     "< $USUAL/_type_.dir/: no fileno");
closedir $DIR                           or die "closedir: $!";
like($DIR_buf, qr/\./xms,   '... can read from directory');
rmdir "$USUAL/_type_.dir"               or die "rmdir: $!";

